import datetime

from twingly_search.errors import TwinglyQueryException

try:
    # python 3
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


class Query(object):
    """
    Twingly Search API Query

    Attributes:
        pattern     (string) pattern the search query
        language    (string) language which language to restrict the query to
        client      (Client) the client that this query is connected to
        start_time  (datetime.datetime) search for posts published after this time (inclusive)
        end_time    (datetime.datetime) search for posts published before this time (inclusive)
    """
    pattern = ''
    language = ''
    client = None
    start_time = None
    end_time = None

    def __init__(self, client):
        """
        No need to call this method manually, instead use {Client#query}.

        :param client: the client that this query should be connected to
        """
        self.client = client

    def url(self):
        """
        :return: request url for the query
        """
        return "%s?%s" % (self.client.endpoint_url(), self.url_parameters())

    def execute(self):
        """
        Executes the query and returns the result
        :return: the result for this query
        """
        return self.client.execute_query(self)

    def url_parameters(self):
        """
        :return: the query part of the request url
        """
        return urlencode(self.request_parameters())

    def request_parameters(self):
        """
        :return: the request parameters
        """
        if len(self.pattern) == 0:
            raise TwinglyQueryException("Missing pattern")

        return {
            'key': self.client.api_key,
            'searchpattern': self.pattern,
            'documentlang': self.language,
            'ts': self._ts(),
            'tsTo': self._tsTo(),
            'xmloutputversion': 2
        }

    def _ts(self):
        if self.start_time is not None:
            if isinstance(self.start_time, datetime.datetime):
                return self.start_time.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(self.start_time, basestring):
                return self.start_time
            else:
                return ''
        else:
            return ''

    def _tsTo(self):
        if self.end_time is not None:
            if isinstance(self.end_time, datetime.datetime):
                return self.end_time.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(self.end_time, basestring):
                return self.end_time
            else:
                return ''
        else:
            return ''



#!/usr/bin/env python

"""A library provides Python interface to the Twingly Search API"""
from __future__ import absolute_import

__author__ = 'Twingly AB'
__version__ = '1.0'

from .client import Client
from .errors import TwinglyException
from .errors import TwinglyAuthException
from .errors import TwinglyQueryException
from .errors import TwinglyServerException
from .parser import Parser
from .post import Post
from .query import Query
from .result import Result



#!/usr/bin/env python
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from twingly_search.post import Post
from twingly_search.result import Result

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

from twingly_search.errors import *


class Parser:
    def parse(self, document):
        """
        Parse an API response body.

        :param document: containing an API response XML
        :return: containing the Result
        """
        try:
            doc = ET.XML(document)
        except Exception as e:
            raise TwinglyServerException(e)

        if doc.tag == 'html':
            raise TwinglyServerException(document)

        if doc.find('{http://www.twingly.com}operationResult') is not None:
            if doc.find('{http://www.twingly.com}operationResult').attrib['resultType'] == 'failure':
                if 'API key' in doc.find('{http://www.twingly.com}operationResult').text:
                    raise TwinglyAuthException(doc.find('{http://www.twingly.com}operationResult').text)
                else:
                    raise TwinglyServerException(doc.find('{http://www.twingly.com}operationResult').text)

        return self._create_result(doc)

    def _create_result(self, data_node):
        result = Result()

        result.number_of_matches_returned = int(data_node.attrib['numberOfMatchesReturned'])
        result.seconds_elapsed = float(data_node.attrib['secondsElapsed'])
        result.number_of_matches_total = int(data_node.attrib['numberOfMatchesTotal'])

        result.posts = []

        for p in data_node.findall('post[@contentType="blog"]'):
            result.posts.append(self._parse_post(p))

        return result

    def _parse_post(self, element):
        post_params = {'tags': []}
        for child in element:
            if child.tag == 'tags':
                post_params[child.tag] = self._parse_tags(child)
            else:
                post_params[child.tag] = child.text

        post = Post()
        post.set_values(post_params)
        return post

    def _parse_tags(self, element):
        tags = []
        for tag in element.find('tag'):
            tags.append(tag.text)
        return tags

    def _handle_failure(self, failure):
        raise TwinglyException.from_api_response_message(failure.text)

    def _handle_non_xml_document(self, document):
        response_text = document.find("//text()").text
        raise TwinglyServerException(response_text)



#!/usr/bin/env python
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os

import requests

from twingly_search import __version__
from twingly_search.errors import *
from twingly_search.parser import Parser
from twingly_search.query import Query


class Client(object):
    """
    Twingly Search API client
    """

    BASE_URL = "https://api.twingly.com"
    SEARCH_PATH = "/analytics/Analytics.ashx"
    DEFAULT_USER_AGENT = "Twingly Search Python Client/%s"

    def __init__(self, api_key=None, user_agent=None):
        """
        :param api_key: (string) Twingly Search API Key
        :param user_agent: (string) User Agent for client
        """

        if api_key is None:
            api_key = self._env_api_key()

        if api_key is None:
            self._api_key_missing()

        self.api_key = api_key

        self._user_agent = user_agent
        if self._user_agent is None:
            self._user_agent = self.DEFAULT_USER_AGENT % __version__

        self._session = requests.Session()
        self._session.headers.update({'User-Agent': self._user_agent})

    def query(self):
        """
        Returns a new <twingly_search.Query> object connected to this client

        :return: Query
        """
        return Query(self)

    def execute_query(self, query):
        """
        Executes the given Query and returns the result

        :param query: the Query to be executed
        :return: Result
        """
        response_body = self._get_response(query).content
        return Parser().parse(response_body)

    def endpoint_url(self):
        """
        :return: API endpoint URL
        """
        return "%s%s" % (self.BASE_URL, self.SEARCH_PATH)

    def _env_api_key(self):
        return os.environ.get('TWINGLY_SEARCH_KEY')

    def _get_response(self, query):
        response = self._session.get(query.url())
        if 200 <= response.status_code < 300:
            return response
        else:
            if response.status_code >= 500:
                raise TwinglyServerException(response.content)
            else:
                raise TwinglyQueryException(response.content)

    def _api_key_missing(self):
        raise TwinglyAuthException("No API key has been provided.")



#!/usr/bin/env python
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

class Result(object):
    """
    Represents a result from a Query to the Search API

    Attributes:
        number_of_matches_returned  (int) number of Post the query returned
        number_of_matches_total     (int) total number of Post the query matched
        seconds_elapsed             (float) number of seconds it took to execute the query
        posts                       (list of Post) all posts that matched the query
    """

    number_of_matches_returned = 0
    seconds_elapsed = 0.0
    number_of_matches_total = 0

    posts = []

    def __init__(self):
        pass

    def all_results_returned(self):
        """
        :return: (boolean) returns True if this result includes all Posts that matched the query
        """
        return self.number_of_matches_total == self.number_of_matches_returned

    def __repr__(self):
        matches = "posts, "
        matches += "number_of_matches_returned=%d, " % self.number_of_matches_returned
        matches += "number_of_matches_total=%d" % self.number_of_matches_total

        return "#<%s:0x%s %s>" % (self.__class__.__name__, id(self), matches)

    def __unicode__(self):
        return "Number of matches returned: %d Seconds elapsed: %.3f Number of matches total: %d" % (
            self.number_of_matches_returned,
            self.seconds_elapsed,
            self.number_of_matches_total
        )

    def __str__(self):
        return self.__unicode__()



#!/usr/bin/env python

'''The setup and build script for the twingly-search library.'''

import os

from setuptools import setup, find_packages

if 'GENERATE_RST' in os.environ:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
else:
    readme_file = open('README.md', 'r')
    long_description = readme_file.read()
    readme_file.close()

setup(
    name='twingly-search',
    version='1.0.3',
    author='Twingly AB',
    author_email='support@twingly.com',
    license='MIT',
    url='https://github.com/twingly/twingly-search-api-python',
    keywords='twingly',
    description='Python library for Twingly Search API',
    long_description=long_description,
    packages=find_packages(exclude=['tests*']),
    install_requires=['future', 'requests'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
)



import datetime

class Post(object):
    """
    A blog post

    Attributes:
         url            (string) the post URL
         title          (string) the post title
         summary        (string) the blog post text
         language_code  (string) ISO two letter language code for the language that the post was written in
         published      (datetime.datetime) published the time, in UTC, when this post was published
         indexed        (datetime.datetime) indexed the time, in UTC, when this post was indexed by Twingly
         blog_url       (string) the blog URL
         blog_name      (string) name of the blog
         blog_rank      (int) the rank of the blog, based on authority and language
                        (https://developer.twingly.com/resources/search/#authority)
         authority      (int) authority the blog's authority/influence
                        (https://developer.twingly.com/resources/search/#authority)
         tags           (list of string) tags
    """
    url = ''
    title = ''
    summary = ''
    language_code = ''
    published = datetime.datetime.strptime("1970-01-01 00:00:00Z", '%Y-%m-%d %H:%M:%SZ')
    indexed = datetime.datetime.strptime("1970-01-01 00:00:00Z", '%Y-%m-%d %H:%M:%SZ')
    blog_url = ''
    blog_name = ''
    authority = 0
    blog_rank = 0
    tags = []

    def set_values(self, params):
        """
        Sets all instance variables for the Post, given a dict.

        :param params: (dict) containing blog post data
        """
        self.url = params['url']
        self.title = params['title']
        self.summary = params['summary']
        self.language_code = params['languageCode']
        self.published = datetime.datetime.strptime(params['published'], '%Y-%m-%d %H:%M:%SZ')
        self.indexed = datetime.datetime.strptime(params['indexed'], '%Y-%m-%d %H:%M:%SZ')
        self.blog_url = params['blogUrl']
        self.blog_name = params['blogName']
        self.authority = int(params['authority'])
        self.blog_rank = int(params['blogRank'])
        self.tags = params['tags']

    def __unicode__(self):
        return "%s %s" % (self.title, self.url)

    def __str__(self):
        return self.__unicode__()



class TwinglyException(Exception):
    def from_api_response_message(self, message):
        """
        :param message: API response error message
        """
        if 'API key' in message:
            raise TwinglyAuthException(message)
        else:
            raise TwinglyServerException(message)


class TwinglyAuthException(TwinglyException):
    pass


class TwinglyServerException(TwinglyException):
    pass


class TwinglyQueryException(TwinglyException):
    pass
